#include "stdio.h"
#include "stdlib.h"
#include "math.h"

/*
	Convert an ascii stl file to the geom format
*/

typedef struct {
	double x,y,z;
} XYZ;

int main(int argc,char **argv)
{
	int i;
	int nf = 0;
	int np = 0;
	char s[256];
	double x,y,z;
	XYZ n,p[3];
	FILE *fptr;

	if (argc < 2) {
		fprintf(stderr,"Usage: %s stlfile\n",argv[0]);
		exit(-1);
	}

	if ((fptr = fopen(argv[1],"r")) == NULL) {
		fprintf(stderr,"Unable to open stl file\n");
		exit(-1);
	}

	fscanf(fptr,"%s",s);
	if (strcmp(s,"solid") != 0) {
		fprintf(stderr,"Expected \"solid\" keyword\n");
		exit(-1);
	}
	ReadLine(fptr,s,256);

	while (fscanf(fptr,"%s",s) == 1) {
		if (strcmp(s,"facet") == 0) {
			if (np == 3) {
				printf("f3");
				for (i=0;i<np;i++)
					printf(" %g %g %g",p[i].x,p[i].y,p[i].z);
				printf(" 1 0 0\n");
				nf++;
			}
			np = 0;
		}

		if (strcmp(s,"normal") == 0) {
			if (fscanf(fptr,"%lf %lf %lf",&n.x,&n.y,&n.z) != 3) {
				fprintf(stderr,"Error reading normal\n");
				exit(-1);
			}
		}

		if (strcmp(s,"vertex") == 0) {
			if (fscanf(fptr,"%lf %lf %lf",&x,&y,&z) != 3) {
				fprintf(stderr,"Error reading vertex\n");
				exit(-1);
			}
			p[np].x = x;
			p[np].y = y;
			p[np].z = z;
			np++;
		}

	}

	fprintf(stderr,"Wrote %d facets\n",nf);
	fclose(fptr);
}

int ReadLine(FILE *fptr,char *s,int lmax)
{
   int i=0,c;

   s[0] = '\0';
   while ((c = fgetc(fptr)) != '\n' && c != '\r') {
      if (c == EOF)
         return(FALSE);
      s[i] = c;
      i++;
      s[i] = '\0';
      if (i >= lmax)
         break;
   }
   return(TRUE);
}

